@@ -279,7 +279,19 @@ QList<QNetworkCookie> Account::lastAuthCookies() const
 
 void Account::clearCookieJar()
 {
-    _am->setCookieJar(new CookieJar);
+    Q_ASSERT(qobject_cast<CookieJar*>(_am->cookieJar()));
+    static_cast<CookieJar*>(_am->cookieJar())->clearSessionCookies();
+}
+
+/*! This shares our official cookie jar (containing all the tasty
+    authentication cookies) with another QNAM while making sure
+    of not loosing its ownership. */
+void Account::lendCookieJarTo(QNetworkAccessManager *guest)
+{
+    auto jar = _am->cookieJar();
+    auto oldParent = jar->parent();
+    guest->setCookieJar(jar); // takes ownership of our precious cookie jar
+    jar->setParent(oldParent); // takes it back
 }
 
 void Account::resetNetworkAccessManager()
@@ -400,6 +412,11 @@ void Account::addApprovedCerts(const QList<QSslCertificate> certs)
     _approvedCerts += certs;
 }
 
+void Account::resetSslCertErrorState()
+{
+    _treatSslErrorsAsFailure = false;
+}
+
 void Account::setSslErrorHandler(AbstractSslErrorHandler *handler)
 {
     _sslErrorHandler.reset(handler);
@@ -498,6 +515,11 @@ void Account::slotHandleErrors(QNetworkReply *reply , QList<QSslError> errors)
         reply->ignoreSslErrors();
     } else {
         _treatSslErrorsAsFailure = true;
+        // if during normal operation, a new certificate was MITM'ed, and the user does not
+        // ACK it, the running request must be aborted and the QNAM must be reset, to not
+        // treat the new cert as granted. See bug #3283
+        reply->abort();
+        resetNetworkAccessManager();
         return;
     }
 }
